package edu.stanford.nlp.parser.ensemble.maltparser.core.symbol;


import edu.stanford.nlp.parser.ensemble.maltparser.core.exception.MaltChainedException;
import edu.stanford.nlp.parser.ensemble.maltparser.core.feature.function.FeatureFunction;
import edu.stanford.nlp.parser.ensemble.maltparser.core.feature.function.Modifiable;
import edu.stanford.nlp.parser.ensemble.maltparser.core.feature.value.FeatureValue;
import edu.stanford.nlp.parser.ensemble.maltparser.core.feature.value.SingleFeatureValue;
import edu.stanford.nlp.parser.ensemble.maltparser.core.symbol.nullvalue.NullValues.NullValueId;

public abstract class TableFeature implements FeatureFunction, Modifiable {
	protected SingleFeatureValue featureValue;
	protected SymbolTable table;
	
	public TableFeature() throws MaltChainedException {
		featureValue = new SingleFeatureValue(this);
	}
	
	public abstract void update() throws MaltChainedException;
	public abstract void initialize(Object[] arguments) throws MaltChainedException;
	public abstract Class<?>[] getParameterTypes();
	
	public String getSymbol(int value) throws MaltChainedException {
		return table.getSymbolCodeToString(value);
	}
	
	public int getCode(String value) throws MaltChainedException {
		return table.getSymbolStringToCode(value);
	}
	
	public SymbolTable getSymbolTable() {
		return table;
	}

	public void setSymbolTable(SymbolTable table) {
		this.table = table;
	}
	
	public void updateCardinality() {
		featureValue.setCardinality(table.getValueCounter()); 
	}
	
	public void setFeatureValue(int value) throws MaltChainedException {
		if (table.getSymbolCodeToString(value) == null) {
			featureValue.setCode(value);
			featureValue.setKnown(table.getKnown(value));
			featureValue.setSymbol(table.getNullValueSymbol(NullValueId.NO_NODE));
			featureValue.setNullValue(true);
		} else {
			featureValue.setCode(value);
			featureValue.setKnown(table.getKnown(value));
			featureValue.setSymbol(table.getSymbolCodeToString(value));
			featureValue.setNullValue(table.isNullValue(value));
		}
	}
	
	public void setFeatureValue(String value) throws MaltChainedException {
		if (table.getSymbolStringToCode(value) < 0) {
			featureValue.setCode(table.getNullValueCode(NullValueId.NO_NODE));
			featureValue.setKnown(table.getKnown(value));
			featureValue.setSymbol(value);
			featureValue.setNullValue(true);
		} else {
			featureValue.setCode(table.getSymbolStringToCode(value));
			featureValue.setKnown(table.getKnown(value));
			featureValue.setSymbol(value);
			featureValue.setNullValue(table.isNullValue(value));
		}
	}
	
	public FeatureValue getFeatureValue() {
		return featureValue;
	}
	
	public boolean equals(Object obj) {
		if (!(obj instanceof TableFeature)) {
			return false;
		}
		if (!obj.toString().equals(this.toString())) {
			return false;
		}
		return true;
	}

	public String getTableName() {
		return table.getName();
	}
	
	public String toString() {
		return table.getName();
	}
}
